@@ -1,4 +1,5 @@
 import asyncio
+import itertools
 import socket
 from asyncio.streams import StreamReader
 from socket import AF_INET, AF_INET6, inet_ntop, inet_pton
@@ -194,10 +195,11 @@ class LocalTCP(asyncio.Protocol):
                     ) from None
                 else:
                     self.remote_tcp = remote_tcp
-                    BIND_ADDR = self.config.BIND_ADDR
-                    _, BIND_PORT = remote_tcp_transport.get_extra_info("sockname")
+                    bind_addr, bind_port = remote_tcp_transport.get_extra_info(
+                        "sockname"
+                    )
                     self.transport.write(
-                        self.gen_reply(SocksRep.SUCCEEDED, BIND_ADDR, BIND_PORT)
+                        self.gen_reply(SocksRep.SUCCEEDED, bind_addr, bind_port)
                     )
                     self.stage = self.STAGE_CONNECT
 
@@ -222,16 +224,17 @@ class LocalTCP(asyncio.Protocol):
                     ) from None
                 else:
                     self.local_udp = local_udp
-                    BIND_ADDR = self.config.BIND_ADDR
-                    _, BIND_PORT = local_udp_transport.get_extra_info("sockname")
+                    bind_addr, bind_port = local_udp_transport.get_extra_info(
+                        "sockname"
+                    )
                     self.transport.write(
-                        self.gen_reply(SocksRep.SUCCEEDED, BIND_ADDR, BIND_PORT)
+                        self.gen_reply(SocksRep.SUCCEEDED, bind_addr, bind_port)
                     )
                     self.stage = self.STAGE_UDP_ASSOCIATE
 
                     self.config.ACCESS_LOG and access_logger.info(
                         f"Established UDP relay for {self.peername} "
-                        f"at {BIND_ADDR,BIND_PORT}"
+                        f"at {bind_addr,bind_port}"
                     )
             else:
                 self.transport.write(self.gen_reply(SocksRep.COMMAND_NOT_SUPPORTED))
@@ -254,6 +257,15 @@ class LocalTCP(asyncio.Protocol):
     def eof_received(self):
         self.close()
 
+    def pause_writing(self) -> None:
+        try:
+            self.remote_tcp.transport.pause_reading()
+        except AttributeError:
+            pass
+
+    def resume_writing(self) -> None:
+        self.remote_tcp.transport.resume_reading()
+
     def connection_lost(self, exc: Optional[Exception]) -> None:
         self.close()
 
@@ -299,6 +311,15 @@ class RemoteTCP(asyncio.Protocol):
     def eof_received(self):
         self.close()
 
+    def pause_writing(self) -> None:
+        try:
+            self.local_tcp.transport.pause_reading()
+        except AttributeError:
+            pass
+
+    def resume_writing(self) -> None:
+        self.local_tcp.transport.resume_reading()
+
     def connection_lost(self, exc):
         self.close()
 
@@ -383,11 +404,13 @@ class LocalUDP(asyncio.DatagramProtocol):
         return RSV, FRAG, ATYP, DST_ADDR, DST_PORT, length
 
     def datagram_received(self, data: bytes, local_host_port: Tuple[str, int]):
-        cond1 = self.host_port_limit in (("0.0.0.0", 0), ("::", 0))
-        cond2 = self.host_port_limit == local_host_port
-        cond3 = self.config.STRICT == False
-        if not (cond1 or cond2 or cond3):
+        cond1 = self.host_port_limit in itertools.product(
+            ("0.0.0.0", "::", local_host_port[0]), (0, local_host_port[1])
+        )
+        cond2 = self.config.STRICT == False
+        if not cond1 and not cond2:
             return
+
         loop = asyncio.get_event_loop()
         loop.create_task(self.relay_task(data, local_host_port))
 
